#include-once

; #INDEX# =======================================================================================================================
; Title .........: _Relay
; AutoIt Version : 3.3.14
; Language ......: English
; Description ...: Control HID interfaced USB controlled Relay
; Author(s):       Paul Verboom (pboom)
;
; Based on code found in the following AutoIt Forum post
; https://www.autoitscript.com/forum/topic/160010-usb-interface-relay-dllcall-help/page/2/
; See following files for samples of relays: AdditionalFiles\USB_HID_Relays.docx
; ===============================================================================================================================

; #CURRENT# =====================================================================================================================
; _RelayGetSerial	- Gets serial number if needed, stored in INI file, Opens DLL & gets handle to Relays
; _RelayScreen		- Briefly activates selected relay places status message
; _RelayOpen		- Turns ON (activates) indicated relay
; _RelayClose		- Turns OFF (de-activates) indicated relay
; _RelayCloseAll	- Turns OFF (de-activates) ALL relays
; ===============================================================================================================================

; #FUNCTION# ====================================================================================================================
; Description:		Gets serial Number of relay enumerate next relay if not in INI file
; Parameters:		RelayName also Section in INI file
; Returns:			Nothing can cause program abort on errors
; Globals			$g_Config			- Configuration to check for serial number and store serial number in
;					$gR_hDllRelay		- Set to Handle for Relay DLL
;					$gR_aRelays[0][2]	- Details on relays, established here so rows can be added by module
;											1st dimension one row per relay
;											2nd dimension values
;												0 = Relay Name also INI Section name
;												1 = Serial number of relay

; Author(s)			Paul verboom (pboom)
;
;Serial number must be specified when opening Handle to relay
;Unique serial numbers allow multiple relay boards to be controlled
;If stored in INI file specified section used stored value OTHERWISE
;Get next unique serial number of board attached and store that into INI file
;Board with most Relays must be first enumerated, to address bug in DLL
;Number of Relays on first board enumerated determines number of relays available on all boards
;USB port used determines order boards are enumerated in.
; ===============================================================================================================================
Func _RelayGetSerial($RelayName)
	Local $bFnd = False
	Local $Serial
	Global $aRet
	IF NOT StringInStr($g_Debug,"r") Then															;Look for serial number in INI file
		$Serial = IniRead($g_Config,$RelayName,'Serial','')											;Unless debug forces renumeration ecah time
		_ArrayAdd($gR_aRelays,$RelayName & '|' & $Serial,0,'|')
		$bFnd = True
	EndIf
	Global $gR_hDllRelay = DllOpen("usb_relay_device.dll")											;Get handle to Relay DLL and retain in Global
	IF StringLen($Serial) = 0 Then																	;No serial number

		$aRet = DllCall( $gR_hDllRelay, "ptr:cdecl", "usb_relay_device_enumerate" )
		If @error Then
		  MsgBox(16, "USB Relay Error", "Unable to enumerate device", 3)
		  Exit
		EndIf

		$ptr = $aRet[0]																				;Get pointer to first instance of data

		WHILE $ptr > 0
			$tag_usb_relay_device_info = "ptr serial_number;ptr device_path;uint relay_type;int relay_next"
			$t_usb_relay_device_info = DllStructCreate( $tag_usb_relay_device_info, $ptr )
			If @error Then
				MsgBox(0, "USB Relay Error", "Error in DLL Structure Creatation:" & @error);
				Exit
			EndIf

			$ptr_serial_number = DllStructGetData( $t_usb_relay_device_info, "serial_number" )		;Get Ptr to serial number
			$tag_serial_number = "char[32]"															; Buffer for serial number for max. 32 characters
			$t_serial_number = DllStructCreate( $tag_serial_number, $ptr_serial_number )			; Fill buffer with data from the memory location given by the pointer
			$Serial = DllStructGetData( $t_serial_number, 1 )										; Get serial number as a string

			$ptr_device_path = DllStructGetData( $t_usb_relay_device_info, "device_path" )			;Get device path
			$tag_device_path = "char[128]"
			$t_device_path = DllStructCreate( $tag_device_path, $ptr_device_path )					; Fill buffer with data from the memory location given by the pointer
			$Path = DllStructGetData( $t_device_path, 1 )

			$relay_type = DllStructGetData( $t_usb_relay_device_info, "relay_type" )				;Get relay type

			$relay_next = DllStructGetData( $t_usb_relay_device_info, "relay_next" )				;Get relay Next

			$i = _ArraySearch($gR_aRelays,$Serial,0,0,0,2,1,1)
			IF $i = -1 Then
				IniWrite($g_Config,$RelayName,'Serial',$Serial)
				IniWrite($g_Config,$RelayName,'Path',$Path)
				IniWrite($g_Config,$RelayName,'Type',$relay_type)									;Relay type only gets reported correctly for first relay detected
				_ArrayAdd($gR_aRelays,$RelayName & '|' & $Serial,0,'|')
				$bFnd = True
				ExitLoop
			EndIf
			$ptr = $relay_next																		;Follow pointer to next relay information
		WEnd
	EndIf
	If NOT $bFnd Then
		MsgBox(0, "USB Relay Error", "Relay not found for :" & $RelayName);
		Exit
	EndIf

EndFunc		;==>_RelayGetSerial

; #FUNCTION# ====================================================================================================================
; Description:		Activates a relay breifly to control Screen Roll Up or Down
;					also sets status message
; Parameters:		$iRelayNumber
;						1	= Screen A Up
;						2	= Screen A Down
;						3	= Screen B Up
;						4	= Screen B Down
; Returns:			Nothing
;					Can cause program abort on errors
;					Causes message in Status
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _RelayScreen($iRelayNumber, $Message, $RelayName)
	_RelayCycle($iRelayNumber, $RelayName)
	$gV_CamStatus = $Message
	$g_bStatus = True
EndFunc		;==>_RelayScreen

; #FUNCTION# ====================================================================================================================
; Description:		Turn ONn Activate a relay
; Parameters:		$Section		- Section ofd INI file with Serial Number of Relay
;					$iChannel		- Relay Number to activate
; Returns:			Nothing
;					Can cause program abort on errors
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _RelayOpen($iChannel, $RelayName)
	Local $Serial																					;Serial number of to activate
	Local $aHandleRelay																				;Handle of relay, first element in array is actual address
	Local $i																						;Locate Relay Serial Number by Relay Name
	$i = _ArraySearch($gR_aRelays,$RelayName,0,0,0,2,1,0)
	IF $i = -1 Then
        MsgBox (16, "USB Relay Error", "Unknown Relay: " & $RelayName, 3)
        Exit
	EndIf
	$Serial = $gR_aRelays[$i][1]
	$aRet = DllCall( $gR_hDllRelay, "int", "usb_relay_init" )										;Initialize the library
	If @error Then
		MsgBox (16, "USB Relay Error", "Cannot initialize Library", 3)
		Exit
	EndIf																							;Get handle to Module with specified serial number
	$aHandleRelay = DllCall( $gR_hDllRelay, "int:cdecl", "usb_relay_device_open_with_serial_number", "str", $Serial, "uint", 5 )
	If @error Then
		MsgBox (16, "USB Relay Error", "Cannot Open device: " & $RelayName, 3)
		Exit
	EndIf
																									;Turn ON Relay
	$aRet = DllCall($gR_hDllRelay, "int:cdecl", "usb_relay_device_open_one_relay_channel", "int", $aHandleRelay[0], "int", $iChannel)
    If @error Then
        MsgBox (16, "USB Relay Error", "The Relay cannot open: " & $RelayName, 3)
        Exit
    EndIf

	DllCall($gR_hDllRelay, "int:cdecl", "usb_relay_device_close", "int", $aHandleRelay[0])			;Close the Handle so other Relay can be opened next
EndFunc		;==>_RelayOpen


; #FUNCTION# ====================================================================================================================
; Description:		Turn OFF Deactivate a relay
; Parameters:		$Section		- Section ofd INI file with Serial Number of Relay
;					$iChannel		- Relay Number to activate
; Returns:			Nothing
;					Can cause program abort on errors
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _RelayClose($iChannel, $RelayName)
	Local $Serial																					;Serial number of to activate
	Local $aHandleRelay																				;Handle of relay, first element in array is actual address
	Local $i																						;Locate Relay Serial Number by Relay Name
	$i = _ArraySearch($gR_aRelays,$RelayName,0,0,0,2,1,0)
	IF $i = -1 Then
        MsgBox (16, "USB Relay Error", "The Relay not intialized: " & $RelayName, 3)
        Exit
	EndIf
	$Serial = $gR_aRelays[$i][1]
	$aRet = DllCall( $gR_hDllRelay, "int", "usb_relay_init" )										;Initialize the library
	If @error Then
		MsgBox (16, "USB Relay Error", "Cannot initialize Library", 3)
		Exit
	EndIf																							;Get handle to Module with specified serial number
	$aHandleRelay = DllCall( $gR_hDllRelay, "int:cdecl", "usb_relay_device_open_with_serial_number", "str", $Serial, "uint", 5 )
	If @error Then
		MsgBox (16, "USB Relay Error", "Cannot Open device: " & $RelayName, 3)
		Exit
	EndIf
																									;Turn OFF the Relay
	$aRet = DllCall($gR_hDllRelay, "int:cdecl", "usb_relay_device_close_one_relay_channel", "int", $aHandleRelay[0], "int", $iChannel)
	If @error Then
        MsgBox (16, "USB Relay Error", "The Relay cannot close: " & $RelayName, 3)
        Exit
    EndIf

	DllCall($gR_hDllRelay, "int:cdecl", "usb_relay_device_close", "int", $aHandleRelay[0])			;Close the Handle so other Relay can be opened next
EndFunc		;==>_RelayClose

; #FUNCTION# ====================================================================================================================
; Description:		Cycle a relay On and OFF
; Parameters:		$RelayName		- Name of Relay to cycle also section of INI file with Serial Number of Relay
;					$iChannel		- Relay Number to activate
; Returns:			Nothing
;					Can cause program abort on errors
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _RelayCycle($iChannel, $RelayName)

	Local $Serial																					;Serial number of to activate
	Local $aHandleRelay																				;Handle of relay, first element in array is actual address
	Local $i																						;Locate Relay Serial Number by Relay Name
	$i = _ArraySearch($gR_aRelays,$RelayName,0,0,0,2,1,0)
	IF $i = -1 Then
        MsgBox (16, "USB Relay Error", "Unknown Relay: " & $RelayName, 3)
        Exit
	EndIf
	$Serial = $gR_aRelays[$i][1]

	$aRet = DllCall( $gR_hDllRelay, "int", "usb_relay_init" )										;Initialize USB Relay library
	If @error Then
		MsgBox (16, "USB Relay Error", "Cannot initialize Library", 3)
		Exit
	EndIf																							;Get handle to Module with specified serial number
	$aHandleRelay = DllCall( $gR_hDllRelay, "int:cdecl", "usb_relay_device_open_with_serial_number", "str", $Serial, "uint", 5 )
	If @error Then
		MsgBox (16, "USB Relay Error", "Cannot Open device: " & $RelayName, 3)
		Exit
	EndIf
																									;Turn on the Relay
	$aRet = DllCall($gR_hDllRelay, "int:cdecl", "usb_relay_device_open_one_relay_channel", "int", $aHandleRelay[0], "int", $iChannel)
    If @error Then
        MsgBox (16, "USB Relay Error", "The Relay cannot open: " & $RelayName, 3)
        Exit
    EndIf

	Sleep(IniRead($g_Config,$RelayName,'delay',500))												;Wait defined period of time
																									;Turn Relay back off
	$aRet = DllCall($gR_hDllRelay, "int:cdecl", "usb_relay_device_close_one_relay_channel", "int", $aHandleRelay[0], "int", $iChannel)
    If @error Then
        MsgBox (16, "USB Relay Error", "The Relay cannot close: " & $RelayName, 3)
        Exit
    EndIf

	DllCall($gR_hDllRelay, "int:cdecl", "usb_relay_device_close", "int", $aHandleRelay[0])			;Close the Handle so other Relay can be opened next
EndFunc		;==>_RelayCycle

; #FUNCTION# ====================================================================================================================
; Description:		Turn OFF Deactivate all relays
; Parameters:		$Section		- Section ofd INI file with Serial Number of Relay
; Returns:			Nothing
;					Can cause program abort on errors
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _RelayCloseAll($RelayName)
	Local $Serial																					;Serial number of to activate
	Local $aHandleRelay																				;Handle of relay, first element in array is actual address
	Local $i																						;Locate Relay Serial Number by Relay Name
	$i = _ArraySearch($gR_aRelays,$RelayName,0,0,0,2,1,0)
	IF $i = -1 Then
        MsgBox (16, "USB Relay Error", "Unknown Relay: " & $RelayName, 3)
        Exit
	EndIf
	$Serial = $gR_aRelays[$i][1]

	$aRet = DllCall( $gR_hDllRelay, "int", "usb_relay_init" )										;Initialize USB Relay library
	If @error Then
		MsgBox (16, "USB Relay Error", "Cannot initialize Library", 3)
		Exit
	EndIf																							;Get handle to Module with specified serial number
	$aHandleRelay = DllCall( $gR_hDllRelay, "int:cdecl", "usb_relay_device_open_with_serial_number", "str", $Serial, "uint", 5 )
	If @error Then
		MsgBox (16, "USB Relay Error", "Cannot Open device: " & $RelayName, 3)
		Exit
	EndIf
																									;Turn OFF All Relays
	$aRet = DllCall($gR_hDllRelay, "int:cdecl", "usb_relay_device_close_all_relay_channel", "int", $aHandleRelay[0], "int", 1)
    If @error Then
        MsgBox (16, "USB Relay Error", "The Relay cannot close: " & $RelayName, 3)
        Exit
    EndIf
	DllCall($gR_hDllRelay, "int:cdecl", "usb_relay_device_close", "int", $aHandleRelay[0])			;Close the Handle so other Relay can be opened next
EndFunc
