#include-once
#include <Timers.au3>								;Time outs when waiting for camera response
#include <AutoItConstants.au3>						;Needed for Hex conversion function

; #INDEX# =======================================================================================================================
; Title .........: _VISCA
; COM Channel ...: 1  These functions all asume Channel 1 is active all other functions must reset serial channel to 1 when done
; AutoIt Version : 3.3.14.5
; Language ......: English
; Description ...: Functions to provide send command to VISCA protocol Cameras
;					For more details see files: AdditionalFiles\PTZ3-RS232.pdf
;												AdditionalFiles\RS232 for VISCA over CAT5 cables using RJ45 connectors2.docx
; Author(s):       Paul Verboom (pboom)
; ===============================================================================================================================

#cs
Provides control of SONY video conference cameras using the VISCA protocol
Special consideration for cameras using 9600 baud RS232 interfaces
Commands are only sent once previous cammands are complete
Commands are only sent if there is a change.
Prevents backup of commands in buffer and lost commands
Keeps cameras responsive.
#ce

; #CURRENT# =====================================================================================================================
; _VSCopen			- Intialize required variables in map variable Open Serial Port determine number of cameras
; _VscPanTiltStop	- Send Pan Tilt Stop command for current camera
; _VscZoomStop		- Send Zoom Stop command for current camera
; _VscPanTilt		- Send Pan tilt command direction and speed for both
; _VscZoom			- Send Zoom command direction and speed
; _VscGetPreset		- Get Pipe seperated Hex values for absolute Pan Tilt Zoom positions
; _VscSetPreset		- Send preset values to currently selected camera as Pipe Seperated Hex Values
; _VscCmdAdd		- Add Visca Command to Queue adding Camera number to expect back
; _VscProcess		- Function called inMain Application loop, Does outgoing and incoming background processing
; _VscSend			- Send a VISCA command used by background processing and directly when getting setting presets
; _VscAddCamNm		- Adds the current camera number to a command VISCA command
; _VscGet			- Waits specified time for a reply to an Inquiry used to get Preset values and number of Cameras.
; _VscInq			- Extract Hex value from a Camera Inquiry Reply Packet
; _VscCmd			- Inserts Hex values into a VISAC command
; ===============================================================================================================================

; #FUNCTION# ====================================================================================================================
; Description:		Opens Serial Port to cameras intiatizes variables in $g_mV
; Parameters:		None
; Returns:			Number of cameras detected
;					0 = no cameras detected or errors attempting to detect cameras
; Globals			$g_Config	- Configuration file to load settings from
;					$gV_		- any variable with this prefix
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _VSCopen()
	Global $gV_bWait = False						;flag indicating waiting for a command to complete
	Global $gV_In = ""								;Used to build Messages returned from VISCA Cameras
	Global $gV_InLast = ""							;Visca returned commands last seen
	Global $gV_CmdCmpl1 = ""						;Command Complete Messages we are currently looking for
	Global $gV_CmdCmpl2 = ""
	Global $gV_CamStatus = ''						;Four values used to compose Status Message During Camera operations
	Global $gV_PanLbl = ''
	Global $gV_TltLbl = ''
	Global $gV_ZoomLbl = ''
	Global $gV_PanCmdLast = ''						;Last PanTilt and Zoom commands sent so to prevent duplicate command
	Global $gV_ZmCmdLast = ''						;from being resent reducing Serial port congestion

	_PortOpen("VISCA")																				;Open Serial Port for Camera Control
	Local $nCamCnt = 0																				;Number of the Cameras being controlled
	_VscSend("88 30 01 FF")																			;Command to determine number cameras daisy chained on port
	Local $nWait = IniRead($g_Config,"VISCA","OpenWait",10)											;seconds to wait for Reply from cameras
	Local $Ack = _VscGet($nWait)																	;wait determined amount of time for reply from cameras
	IF @error Then																					;Check for any errors in communication
		MsgBox($MB_ICONERROR,$g_Title,"Unable to Communicate with Cameras" & @CRLF & "Program will terminate")
		_Commcloseport(true)
		Exit
	EndIf
	_DebugMess("a","ACK Message: " & $Ack)															;for debugging
	$iCamFnd = Dec(_VscInq("88 30 0Y FF","Y",$Ack))-1												;Retrieve number of cameras detected
	Return $iCamFnd
EndFunc ;==>_VSCopen

; #FUNCTION# ====================================================================================================================
; Description:		Stops PanTilt for current camera
; Parameters:		None
; Returns:			Nothing
; Author(s)			Paul verboom (pboom)
;
; Sends VISCA Pan Tilt Stop Command
; ===============================================================================================================================
Func _VscPanTiltStop()
	_VscCmdAdd(_VscAddCamNm("8x 01 06 01 00 00 03 03 FF"))											;Set up Pan Stop command to issue after next ACK wait if any completes
	$gV_PanCmdLast = "8x 01 06 01 00 00 03 03 FF"													;set this as the last command
EndFunc

; #FUNCTION# ====================================================================================================================
; Description:		Stops Zoom for current camera
; Parameters:		None
; Returns:			Nothing
; Author(s)			Paul verboom (pboom)
;
; Sends VISCA ZOOM Stop Command
; ===============================================================================================================================
Func _VscZoomStop()
	_VscCmdAdd(_VscAddCamNm("8x 01 04 07 00 FF"))													;Set up ZOOM Stop command to issue after next ACK wait if any completes
	$gV_ZmCmdLast = "8x 01 04 07 00 FF"																;set this as the last command
EndFunc

; #FUNCTION# ====================================================================================================================
; Description:		Sets Pan Tilt in motion in direction and speed indicated
; Parameters:		$PanDir 	- Left or Right only first character read any other character stopped
;					$iPanSpd 	- 0 through 100 converted to Pan speed range of camera 0 - Hex18, if 0 stop direction issued
;					$TltDir 	- Up or Down only first caharacter read any other character stopped
;					$iTltSpd 	- 0 through 100 converted to Tilt speed range of camera 0 - Hex14, if 0 stop direction issued
; Returns:			Nothing
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _VscPanTlt($PanDir,$iPanSpd,$TltDir,$iTltSpd)

	$VV = INT(Dec('18') * $iPanSpd /100 )															;calculate Pan speed 00-18
	$gV_PanLbl = 'Pan ' & $PanDir & ' ' & $VV
	IF StringLeft($PanDir,1) = 'L' Then 															;Determine Pan direction
		$LR = 01																					;Left = 01
	ElseIf StringLeft($PanDir,1) = 'R' Then 														;Right = 02
		$LR = 02																					;Stop = 03
	Else
		$LR = 03
		$gV_PanLbl = "Pan Stopped"
	EndIf
	IF $VV = 0 Then 																				;IF Speed is zero than direction is stop
		$LR = 03
		$gV_PanLbl = "Pan Stopped"
	EndIf

	$WW = INT(Dec('14') * $iTltSpd / 100)															;calculate Tilt speed 01-14
	$gV_TltLbl = 'Tilt ' & $TltDir & ' ' & $WW
	IF StringLeft($TltDir,1) = 'U' Then 															;Determine Tilt direction
		$UD = 01																					;Up = 01
	ElseIF StringLeft($TltDir,1) = 'D' Then															;Down = 02
		$UD = 02																					;Stop = 03
	Else
		$UD = 03
		$gV_TltLbl = "Tilt Stopped"
	EndIf
	IF $WW = 0 Then 																				;IF Speed is zero than direction is stop
		$UD = 03
		$gV_TltLbl = "Tilt Stopped"
	EndIf

	$PanCmd = "8x 01 06 01 VV WW LR UD FF"
	$PanCmd = StringReplace($PanCmd,"VV",Hex($VV, 2))												;Insert Pan speed
	$PanCmd = StringReplace($PanCmd,"WW",Hex($WW, 2))												;Insert Tilt speed
	$PanCmd = StringReplace($PanCmd,"LR",Hex($LR, 2))												;Insert Pan Direction
	$PanCmd = StringReplace($PanCmd,"UD",Hex($UD, 2))												;Insert Tilt Direction
	$PanCmd = _VscAddCamNm($PanCmd)
	IF Not($PanCmd  == $gV_PanCmdLast) Then															;has the actual command changed only take time to send if it has
		_VscCmdAdd($PanCmd)																			;Now Que up command to send
		$gV_PanCmdLast = $PanCmd																	;Last Pan Command so we can tell if mouse position change resulted in new command
		$g_bStatus = True
	EndIf
EndFunc

; #FUNCTION# ====================================================================================================================
; Description:		Sets Zoom action in direction and speed indicated
; Parameters:		$ZmDir 	- Tele or Wide only first caharacter read
;					$iZmSpd - Speed of Zoom Action 0 through 100 converted to Zoom speed range of camera 2 - 7 if 0 zoom stopped
; Returns:			Nothing
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _VscZoom($ZmDir,$iZmSpd)
	$Z = INT(6 * $iZmSpd /100 )																		;determine ZOOm speed 2-7
	IF $Z > 0 Then $Z += 1
	$gV_ZoomLbl = 'Zoom ' & $ZmDir & ' ' & $Z
	IF StringLeft($ZmDir,1) = 'T' Then  															;Determine Zoom Tele=In Wide=out
		$T = 2																						;Tele = 2
	Else																							;Wide = 3
		$T = 3																						;Stop = 0
	EndIf
	IF $Z = 0 Then 																					;IF Speed is zero than Zoom mode is stop
		$T = 0
		$gV_ZoomLbl = "Zoom Stopped"
	EndIf
	$ZoomCmd = "8x 01 04 07 TZ FF"
	$ZoomCmd = StringReplace($ZoomCmd,"T",Hex($T, 1))												;Insert Pan speed
	$ZoomCmd = StringReplace($ZoomCmd,"Z",Hex($Z, 1))												;Insert Tilt speed
	$ZoomCmd = _VscAddCamNm($ZoomCmd)
	IF Not($ZoomCmd  == $gV_ZmCmdLast) Then															;has the actual command changed only take time to send if it has
		_VscCmdAdd($ZoomCmd)																		;Now Que up command to send
		$gV_ZmCmdLast = $ZoomCmd																	;Last Pan Command so we can tell if mouse position change resulted in new command
		$g_bStatus = True
	EndIf
EndFunc

; #FUNCTION# ====================================================================================================================
; Description:		Get Values for Preset
; Parameters:		None
; Globals			$iCam	- Number of currently selected camera
; Returns:			Pipe seperated Hex values for absolute Pan Tilt Zoom positions
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _VscGetPreset()
	Local $Mess
	_VscSend('8x 09 06 12 FF')																		;Pan/Tilt Position inquiry
	$Mess = _VscGet(2,$iCam)																		;wait 2 seconds and for the reply Message and check camera number
	IF @error Then Return ''																		;error with inquiry so return null
	Local $Pan = _VscInq('Y0 50 0W 0W 0W 0W 0Z 0Z 0Z 0Z FF','W',$Mess)								;Extract absolute Pan position from return message
	Local $Tilt = _VscInq('Y0 50 0W 0W 0W 0W 0Z 0Z 0Z 0Z FF','Z',$Mess)								;Extract absolute Title position from return message
	_VscSend('8x 09 04 47 FF')																		;Zoom position enquiry
	$Mess = _VscGet(2,$iCam)																		;wait 2 seconds and for the reply Message and check camera number
	IF @error Then Return ''																		;error with inquiry so return null
	Local $Zoom = _VscInq('Y0 50 0Z 0Z 0Z 0Z FF','Z',$Mess)											;Extract absolute ZOOM position from return message
	Local $StrdPrst = $Pan & '|' & $Tilt & '|' & $Zoom												;Prepare line to use to insert/add
	Return $StrdPrst																				;Preset Name still needs to be added
EndFunc

; #FUNCTION# ====================================================================================================================
; Description:		Send preset values to currently selected camera
; Parameters:		Single String with Pipe seperated values
; Globals			$iCam	- Number of currently selected camera
; Returns:			Nothing
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _VscSetPreset($StrdPrst)
	$aPresetVal = StringSplit($StrdPrst,'|')														;Split string into array
	IF $aPresetVal[0] < 3 Then Return SetError(1)													;element 0 is number of elements we need 0 plus 3
	$PanCmd = "8x 01 06 02 VV WW 0Y 0Y 0Y 0Y 0Z 0Z 0Z 0Z FF"										;Pan/Tilt absolute command
	$PanCmd = StringReplace($PanCmd,"VV",Hex(18, 2))												;Insert Max Pan speed
	$PanCmd = StringReplace($PanCmd,"WW",Hex(14, 2))												;Insert Max Tilt speed
	$PanCmd = _VscCmd($PanCmd,'Y',$aPresetVal [1])													;Insert HEX Pan/Tilt values into Command mask
	$PanCmd = _VscCmd($PanCmd,'Z',$aPresetVal[2])

	$ZoomCmd = "8x 01 04 47 0Z 0Z 0Z 0Z FF"															;Zoom Absolute Command
	$ZoomCmd = _VscCmd($ZoomCmd,'Z',$aPresetVal[3])													;insert HEX Zoom values into Command mask

	_VscCmdAdd($PanCmd)																				;Now Que up commands to send
	_VscCmdAdd($ZoomCmd)
EndFunc
; #FUNCTION# ====================================================================================================================
; Description:		Add Visca Command to Queue adding Camera number
; Parameters:		$Cmd				- VISCA command to Add to Queue
; Globals			$iCam	- Number of currently selected camera
; Returns:			Nothing
; Globals:			$g_Debug	d - trace out VISCA Command added to Queue
;
; newest commands inserted at start of array element 0 pushing all others upward
; Buffer must clear before $gV_bWait clears and Selected Camera can change
; Uses Global Array $g_aVscCmd that all VISCA functions can access
; Each command is proceeded with the Camera number in X| formate where X is the Camera number plus 8
; ===============================================================================================================================
Func _VscCmdAdd($Cmd)
	IF StringInStr($g_Debug,"1") Then 																;for debugging allow Camera to be forced too
		$iCamAct = 1																				;camera 1
	Else
		$iCamAct = $iCam
	EndIf
	$Cmd = HEX($iCamAct+8,1) & '~' & $Cmd															;proceed VISCA command with camera number
	IF UBOUND($g_aVscCmd) = 0 Then																	;we are waiting fro reply from
		_ArrayAdd($g_aVscCmd,$Cmd)
	Else
		_ArrayInsert($g_aVscCmd,0,$Cmd)
	EndIf
	_DebugMess('d','VISCA Command added to Queue: ' & $Cmd)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description:		Main loop processing of VISCA commands
; Parameters:		None
; Returns:			Nothing
; Globals:			$g_Debug	i - incoming Visca Commands
;								c - command complete Messages to look for
;
; Add to main processing Loop for GUI
; Sends Queued VISCA commands when Wait has cleared.
; Builds response Messages used to Clear wait
; ===============================================================================================================================
Func _VscProcess()
    While True																						;Read Messages back from camera
		$NextChar = _CommReadByte()																	;loop to read in command complete messages from camera
		IF @error Then ExitLoop																		;Need to see command complete command before
		$gV_In = $gV_In & Hex($NextChar,2) & " "													;sending any more commands
	WEnd
	IF StringInStr($g_Debug,'i') Then 																;Only look for changes in returned ACK in string if
		IF $gV_In <> $gV_InLast Then																;debugging of ACK to console is enabled
			_DebugMess('i',$gV_In)																	;saves time
		EndIf
		$gV_InLast = $gV_In
	EndIf

	IF $gV_bWait Then																				;If we are waiting  check for command complete recieved
		IF StringInStr($gV_In,$gV_CmdCmpl1,$STR_NOCASESENSEBASIC) OR StringInStr($gV_In,$gV_CmdCmpl2,$STR_NOCASESENSEBASIC) Then
			$gV_bWait = False																		;Clear AckWait flag allows more pan commands to be queued
			$g_bStatus = True
			$gV_In = ""																				;Command Complete recieved now clear Visca so we don't this instance again
		EndIf
	EndIf

	IF (NOT $gV_bWait) AND UBOUND($g_aVscCmd) > 0 Then												;Any Command in Que to send
		$VscCmd = _ArrayPop($g_aVscCmd)																;Send command from end of array, new commands inserted into start of array
		$gV_CmdCmpl1 = StringReplace("z0 51 FF","z",StringLeft($VscCmd,1),1,1)						;set Command Complete acknowledgement we need to see before send more PAN commands
		$gV_CmdCmpl2 = StringReplace("z0 52 FF","z",StringLeft($VscCmd,1),1,1)
		$VscCmd = StringMid($VscCmd,3)																;Trim off the camera number from the command
		_VscSend($VscCmd)																			;Send command from end of array, new commands inserted into start of array
		_DebugMess("c","Desired Command Complete Message: " & $gV_CmdCmpl1)							;for debugging
		$gV_bWait = True																			;set up to wait for ACK
		$g_bStatus = True
	EndIf
EndFunc

; #FUNCTION# ====================================================================================================================
; Description:		Sends a VISCA command
; Parameters:		$VscHex		- Bytes to send expressed as a ASCII Hex string ie: '88 30 04 FF'
; Returns:			Nothing
; Globals			$iCam	- Number of currently selected camera
; Author(s)			Paul verboom (pboom)
;
; Function causes an program abort should command to send be improperly formatted
; Will update Camera Status line with any errors
; ===============================================================================================================================
Func _VscSend($VscHex)
	IF StringInStr($g_Debug,"1") Then 																;for debugging allow Camera to be forced too
		$iCamAct = 1																				;camera 1
	Else
		$iCamAct = $iCam
	EndIf
	$VscHex = StringReplace($VscHex,"x",HEX($iCamAct,1),1,1)										;Insert camera number in string
	_DebugMess("v","Visca Command Sent: " & $VscHex)												;Trace out the Visca Command about to be sent
	Local $aBytes = StringSplit($VscHex," ")														;provide string as shown in documentation ie. 88 30 01 FF
	Local $nBytes = $aBytes[0]																		;covert individual values into string array
	IF $nBytes = 0 Then Exit																		;No values then just exit with no error
	$Struct1 = DllStructCreate("byte[" & $nBytes & "]")												;Create an array to pass to the DLL allows zero bytes to be sent
	For $n = 1 To $nBytes 																			;Put the bytes into the DLL data struct
		$nByte = Dec($aBytes[$n], $NUMBER_AUTO)														;Convert individual HEX values in interger values
		IF $nByte < 0 OR $nByte > 255 Then 															;verify all byte values are in range
			MsgBox($MB_ICONERROR,$g_Title,"Invalid Byte value in VISCA command: " & $VscHex)		;errors here are only caused internally by code not formatting commands properly
			Exit																					;so we aboty program
		EndIf
		DllStructSetData($Struct1, 1, $nByte, $n)													;Place Byte into Array sent to DLL
	Next
	_CommSendByteArray(DllStructGetPtr($Struct1), $nBytes, 0)										;send the bytes
	IF @error Then 																					;error sending the bytes
		$gV_PanLbl = ''																				;Clear messages
		$gV_TltLbl = ''																				;Set an Error message
		$gV_ZoomLbl = ''
		$gV_CamStatus = 'Error sending VISCA Command'
		$g_bStatus = True
	EndIf
EndFunc

; #FUNCTION# ====================================================================================================================
; Description:		Add Camera Number to Visca Command string
; Parameters:		$VscHex		- Command to insert camera number in as a ASCII Hex string ie: '88 30 04 FF'
; Returns:			Command with current camera number inserted
; Globals:			$iCam  - Camera Number
;
; Replaces 'x' & 'z' place markers with the camera number 1-7
; z is for Command Complete message and has 8 added to it
; ===============================================================================================================================
Func _VscAddCamNm($VscHex)
	IF StringInStr($g_Debug,"1") Then 																;for debugging allow Camera to be forced too
		$iCamAct = 1																				;camera 1
	Else
		$iCamAct = $iCam
	EndIf
	$VscHex = StringReplace($VscHex,"x",HEX($iCamAct,1),1,1)										;Insert camera number in string (x = camera number)
	$VscHex = StringReplace($VscHex,"z",HEX($iCamAct+8,1),1,1)										;Insert camera number for ACK message (z = address +8)
	Return $VscHex
EndFunc


; #FUNCTION# ====================================================================================================================
; Description:		Wait for VISCA Inquiry return message
; Parameters:		$iDelaySeconds		- Delay in seconds to wait for message, default is 2 seconds
;					$iCamNumber			- Camera message is expected from,default is 0 which equals any
;Returns:			Returned Inquiry message
;					Sets @Error on any error after displaying error message
;Globals:			$g_Debug			- 'a' Debug out returned Inquiry Message
; ===============================================================================================================================
Func _VscGet($iDelaySeconds = 2,$iCamNumber = 0)													;Default delay is 2 seconds, Verify Camera number if value <> 0
	Local $Mess 																					;Any Error Message
	Local $iDelay = $iDelaySeconds * 1000															;How long to wait for reply
	Local $hTimer = TimerInit()																		;Start a timer
	Local $VscTrm = Dec("FF", $NUMBER_AUTO)															;The terminator Byte for all Visca commands and returned messages
	Local $VscHex = ''																				;used to build the response

	While True																						;loop to read in ACK messages from camera
		$NextChar = _CommReadByte(0)																;Get a byte a string conversion of number (eg '0', '20') with no waiting
		IF @error Then																				;@error = 1 No character recieved
			IF @error = 2 Then
				$Mess = "DLL Call failed"
				ExitLoop
			EndIf
			IF (_Timer_Diff($hTimer)) > $iDelay Then												;check on time limit
				$Mess = "Command reception timeout"
				ExitLoop
			EndIf
		Else
			$VscHex = $VscHex & Hex($NextChar,2) & " "
			IF $NextChar = $VscTrm Then ExitLoop													;Inquiry response terminator charcater recieved
			IF StringLen($VscHex) > 60 Then
				$Mess = "To Many Characters recieved"
				ExitLoop
			EndIf
		EndIf
	WEnd
	IF $iCamNumber > 0 AND StringLen($Mess) = 0 AND NOT StringInStr($g_Debug,'n') Then				;Verify return message is from expected camera
		IF Dec(StringMid($VscHex,1,1)) <> $iCamNumber + 8 Then										;if not in debug mode and no error so far
			$Mess = $Mess & @CRLF & "Inquiry Response from wrong camera"
		EndIf
	EndIf
	IF StringLen($Mess) > 0 Then
		$Mess = 'Camera Communications Error' & @CRLF & $Mess										;Add first common line to error
		MsgBox($MB_ICONERROR,$g_Title,$Mess)														;Display the Message
		SetError(1)
	EndIf
	_DebugMess("a","Inquiry Message: " & $VscHex)													;Debug the Inquiry Message
	Return $VscHex																					;Return the string in HEX format
EndFunc

; #FUNCTION# ====================================================================================================================
; Description:		Extract HEX value from returned Inquiry Message
; Parameters:		$Pattern			- Sample of the Inquiry Command Reply Packet, can be taken directly from VISCA command reference
;					$Marker				- Letters that represent HEX characters in $Pattern
;					$Message			- Inquiry Command Reply Packet to extract values from
; Returns:			Hex Value as ASCI ie: 'FF01'
;
; Used to extract HEX represntation of camera positions for example
; Packet Reply: Y0 50 0Z 0Z 0Z 0Z FF
; where the four Z characters are the 4 digit hex value
; ===============================================================================================================================
Func _VscInq($Pattern,$Marker,$Message)																;Extract Hex Values from Inquiry Packet Reply
	Local $nPos = 0																					;Position of value to extract in $Message
	Local $nOccr = 1																				;is determined by position of $Marker in $Pattern
	Local $HexOut = ""																				;Marker can be on or several characters
	Local $nMkLn = StringLen($Marker)
	While True																						;Marker can occur Multiple times
		$nPos = StringInstr($Pattern,$Marker,$STR_NOCASESENSEBASIC,$nOccr)							;Pattern is provided in same format as Documentation
		IF $nPos > 0 Then
			$HexOut = $HexOut & StringMid($Message,$nPos,$nMkLn)
			$nOccr += 1
		Else
			ExitLoop
		EndIf
	WEnd
	Return $HexOut
EndFunc

; #FUNCTION# ====================================================================================================================
; Description:		Insert HEX values into a VISCA command
; Parameters:		$Cmd				- VISCA command to insert Hex values into
;					$Marker				- Letters that represent HEX characters can be anywhere in $Cmd continous or not continous
;					$Hex				- Hex value to insert, will be insert one character at a time must not have spaces
; Returns:			VISCA command with inserted value
;
; Used to insert HEX values into commands such as setting absolute value in Direct ZOOm command
; Packet Reply: Y0 50 0Z 0Z 0Z 0Z FF
; where the four Z characters are replaced with 4 digit hex value 0000 to 03FFF
; ===============================================================================================================================
Func _VscCmd($Cmd,$Marker,$Hex)
	Local $Tmp2 = StringReplace($Cmd,$Marker,$Marker)													;Determine number of markers in Command
	Local $nCntChr = @extended
	$Hex = Hex(Dec($hex),$nCntChr)																		;Reformat HEX value so it is correct number of digits, will add leading zeros
	FOR $n = 1 to $nCntChr
		$Cmd = StringReplace($Cmd,$Marker,StringMid($Hex,$n,1),1)
	Next
	Return $Cmd
EndFunc

